# -*- coding: utf-8 -*-


from sqlalchemy import create_engine
from pyramid.compat import text_type
from pyramid_basemodel import Base
from pyramid_basemodel import Session

from tests import BaseRequestTest
from pyramid_localize.models import Language
from pyramid_localize.request import locale_id
from pyramid_localize.request import locales


class RequestTest(BaseRequestTest):

    def test_request(self):
        '''Test whether route-parameters gets filled'''
        request = self._makeRequest()
        route_parameters = request.default_locale(slug='some-slug')
        self.assertTrue('__LOCALE__' in route_parameters)
        self.assertTrue(route_parameters['__LOCALE__'] in
                        request.config.localize.locales.available)

    def test_filled(self):
        '''Test whether correct __LOCALE__ is left'''
        request = self._makeRequest()
        route_parameters = request.default_locale(slug='some-slug', __LOCALE__='pl')
        self.assertEqual(route_parameters['__LOCALE__'], 'pl')

    def test_filled_wrong(self):
        '''Test whether wrong locale is change'''
        request = self._makeRequest()
        route_parameters = request.default_locale(slug='some-slug', __LOCALE__='fr')
        self.assertTrue(route_parameters['__LOCALE__'] in
                        request.config.localize.locales.available)

engine = create_engine('sqlite://', echo=False)
Session.configure(bind=engine)
Base.metadata.bind = engine


class RequestMethodTest(BaseRequestTest):

    def setUp(self):
        '''
            setUp test method @see unittest.TestCase.setUp
        '''
        Base.metadata.create_all(engine)

        for locale in ['pl', 'cz', 'fr']:
            locale_object = Language(name=text_type(locale),
                                     native_name=text_type(locale),
                                     language_code=text_type(locale))
            Session.add(locale_object)

    def tearDown(self):
        '''
            This tears tests down
        '''
        Base.metadata.drop_all(engine)

    def test_locale_id(self):
        '''Test for creating, and getting loacel id'''
        request = self._makeRequest()
        lid = locale_id(request)
        self.assertTrue(isinstance(lid, int))

    def test_locales(self):
        '''test return locales list'''
        request = self._makeRequest()
        locales_list = locales(request)
        self.assertEqual(3, len(locales_list))

    def test_locales_config(self):
        '''test return locales list limited by config'''
        request = self._makeRequest()
        locales_list = locales(request, True)
        self.assertEqual(4, len(locales_list))



# -*- coding: utf-8 -*-

import sys
import os
import unittest
from mock import Mock

from pyramid.path import package_path
from pyramid.request import Request
from pyramid.i18n import Localizer
from pyramid import testing

from pyramid_localize.tools import dummy_autotranslate
from pyramid_localize.tools import destination_path
from pyramid_localize.tools import locale_negotiator
from pyramid_localize.tools import set_localizer


class SetLocalizerTests(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def _makeRequest(self, environ=None):
        if environ is None:
            environ = {}
        request = Request(environ)
        request.registry = self.config.registry
        request.config = Mock()
        mock_configuration = {
            'localize.locales.available': ['en', 'pl', 'de', 'cz']}
        request.config.configure_mock(**mock_configuration)
        set_localizer(request)
        return request

    def test_simple(self):
        '''simple localizer setting test'''
        request = self._makeRequest()
        self.assertTrue(isinstance(request.localizer, Localizer))
        self.assertTrue(hasattr(request, '_'))

    def test_reset(self):
        '''test resetting localizer capabilites'''
        request = self._makeRequest()
        old_localizer = request.localizer
        self.assertEqual(old_localizer, request.localizer)
        set_localizer(request, reset=True)
        self.assertNotEqual(old_localizer, request.localizer)

    def test_translate(self):
        '''simple test for translating method call'''
        request = self._makeRequest()
        msgid = 'Test message'
        self.assertEqual(msgid, request._(msgid))


class LocaleNegotiatorTests(unittest.TestCase):

    def setUp(self):

        self.request = Mock()
        mock_configuration = {
            'config.localize.locales.available': ['en', 'pl', 'de', 'cz'],
            'config.localize.locales.default': 'en',
            'cookies': {'lang': 'cz'},
            'accept_language.best_match.return_value': 'de',
            'path': '/pl/page'}
        self.request.configure_mock(**mock_configuration)

    def test_negotiate_path(self):
        '''locale_negotiator:path'''
        locale = locale_negotiator(self.request)

        self.assertEqual(locale, 'pl')

    def test_negotiate_cookie(self):
        '''locale_negotiator:cookie'''
        self.request.path = '/page'
        locale = locale_negotiator(self.request)

        self.assertEqual(locale, 'cz')

    def test_negotiate_headers(self):
        '''locale_negotiator:header
        That's more like a webob job, should be tested there
        '''

        self.request.path = '/page'
        self.request.cookies = {}
        locale = locale_negotiator(self.request)

        self.assertEqual(locale, 'de')

    def test_negotiate_default(self):
        '''locale_negotiator:default'''

        self.request.path = '/page'
        self.request.cookies = {}
        self.request.accept_language = None
        locale = locale_negotiator(self.request)

        self.assertEqual(locale, 'en')


class DestinationPathTests(unittest.TestCase):

    def test_filename(self):
        '''testing translation fullpath resolve'''
        request = Mock()
        path = '/some/path/to/translations'
        mock_configuration = {
            'config.localize.translation.destination': path}
        request.configure_mock(**mock_configuration)
        result = destination_path(request)
        self.assertEqual(result, path)

    def test_package(self):
        '''testing translation package:path resolve'''
        request = Mock()
        mock_configuration = {
            'config.localize.translation.destination': 'tests:translations'}
        request.configure_mock(**mock_configuration)
        result = destination_path(request)
        self.assertEqual(result,
                         os.path.join(package_path(sys.modules['tests']),
                                      'translations'))


class DummyTranslationTests(unittest.TestCase):

    def test_message(self):
        '''dummy_autotranslate::simple'''
        text = 'Simple fake text'
        translated_text = dummy_autotranslate(text)
        self.assertEqual(text, translated_text)

    def test_default(self):
        '''dummy_autotranslate::default'''
        text = 'Simple fake text'
        translated_text = dummy_autotranslate('test-msgid', default=text)
        self.assertEqual(text, translated_text)

    def test_replace(self):
        '''dummy_autotranslate::default'''
        text = 'Simple ${what} text'
        translated_text = dummy_autotranslate(text, mapping={'what': 'fake'})
        self.assertEqual('Simple fake text', translated_text)



# -*- coding: utf-8 -*-

import unittest
from mock import Mock

from pyramid.request import Request
from pyramid.decorator import reify

from pyramid_localize.request import LocalizeRequestMixin
from pyramid_localize.request import locale
from pyramid_localize.request import database_locales


class TestRequest(Request, LocalizeRequestMixin):

    @reify
    def locale(self):
        return locale(self)

    @reify
    def _database_locales(self):
        return database_locales(self)


class BaseRequestTest(unittest.TestCase):

    def _makeRequest(self, environ=None):
        if environ is None:
            environ = {}
        request = TestRequest(environ)
        request.config = Mock()
        mock_configuration = {
            'localize.locales.available': ['en', 'pl', 'de', 'cz']}
        request.config.configure_mock(**mock_configuration)
        return request



# -*- coding: utf-8 -*-

import pyramid.request
from pyramid.i18n import get_locale_name
from pyramid.compat import text_type
from pyramid_basemodel import Session
from pyramid_localize.models import Language


class LocalizeRequestMixin(object):

    def default_locale(self, **kw):
        '''
            Sets up default locale for path kwargs. Can be used in custom route_url overwrites

            :param kwargs kw: list of route parts

            :returns: kw
        '''
        if '__LOCALE__' not in kw or\
                kw['__LOCALE__'] not in self.config.localize.locales.available:
            kw['__LOCALE__'] = self.locale

        return kw

    def route_url(self, route_name, *elements, **kw):  # pragma: no cover
        '''
            Overwrites original route_url to handle default locale

            .. note:: see :meth:`pyramid.request.Request.route_url`
        '''

        return pyramid.request.Request.route_url(self, route_name, *elements, **self.default_locale(**kw))


def locale(request):  # pragma: no cover
    '''
        When called for the first time, it ask enviroment for languagecode, which is later available as a pure property
        overriding this method

        :returns: language code needed for translations
        :rtype: string
    '''
    return get_locale_name(request)


def locale_id(request):
    '''
        Returns database id of a current locale name

        :returns: database id of a language code needed for translations
        :rtype: int
    '''

    if not request.locale in request._database_locales:
        _create_locale(request.locale, request)

    return request._database_locales[request.locale].id


def database_locales(request):
    '''
        Returns list of all database locales available

        :returns: dictionary of Language objects language_code: Language
        :rtype: dict
    '''
    locales = {}
    for language in Session.query(Language).all():
        locales[language.language_code] = language

    return locales


def locales(request, config=False):
    '''
        Returns a list of locales

        :param bool config: Whether to restrict list with config

        :returns: dictionary of Language objects language_code: Language
        :rtype: dict
    '''
    if config:
        locales = {}
        for locale in request.config.localize.locales.available:
            if not locale in request._database_locales:
                _create_locale(locale, request)
            locales[locale] = request._database_locales[locale]

        return locales

    return request._database_locales


def _create_locale(locale, request):
    new_locale = Language(name=text_type(locale),
                          native_name=text_type(locale),
                          language_code=text_type(locale))
    Session.add(new_locale)
    request._database_locales = database_locales(request)



# -*- coding: utf-8 -*-
'''
    methods in this module are tools, thank to which pyramid_localize works most of its magic
'''
import sys
import os
import logging

from translationstring import _interp_regex
from pyramid.i18n import get_localizer
from pyramid.i18n import make_localizer
from pyramid.i18n import TranslationString
from pyramid.asset import resolve_asset_spec
from pyramid.path import package_path
from pyramid.httpexceptions import HTTPNotFound
from pyramid.interfaces import ILocalizer
from pyramid.interfaces import ITranslationDirectories
from pyramid.compat import text_type

logger = logging.getLogger(__name__)


def set_localizer(request, reset=False):
    '''
        Sets localizer and auto_translate methods for request

        :param pyramid.request.Request request: request object
        :param bool reset: flag that directs resetting localizer within app
    '''

    if reset:

        request.localizer = None
        for locale in request.config.localize.locales.available:
            logger.debug('Resetting {0} localizator'.format(locale))
            tdirs = request.registry.queryUtility(ITranslationDirectories,
                                                  default=[])
            localizer = make_localizer(locale, tdirs)
            request.registry.registerUtility(localizer, ILocalizer,
                                             name=locale)

    localizer = get_localizer(request)

    def auto_translate(*args, **kwargs):
        # lets pass default domain, so we don't have to determine it with
        # each _() function in apps.
        if len(args) <= 1 and not 'domain' in kwargs:
            kwargs['domain'] = request.config.localize.domain

        # unlike in examples we use TranslationString, to make sure we always
        # use appropriate domain
        return localizer.translate(TranslationString(*args, **kwargs))

    request.localizer = localizer
    request._ = auto_translate


def locale_negotiator(request):
    '''
        Locale negotiator. It sets best suited locale variable for given user:

        1. Tries the address url first, if the first part has locale indicator.
        2. It checks cookies, for value set here
        3. Tries to best match accepted language for browser user is visiting website with
        4. Defaults to en

        :param pyramid.request.Request request: a request object
        :returns: locale name
        :rtype: str

    '''
    available_languages = request.config.localize.locales.available
    locale = request.config.localize.locales.default
    # We do not have a matchdict present at the moment, lets get our own split
    # (request.path is always a /, so we'll get two elements)
    route_elements = request.path.split('/')
    # we check if route_element[1] is a locale indicator for path
    if len(route_elements[1]) == 2 and route_elements[1] in available_languages:
        locale = route_elements[1]
        # TODO: Code below is a wish,. After detecting locale on url, remove it and progress with standard url name
        # route_elements.remove(locale)
        # request.set_property(lambda request: '/'.join(route_elements), name='path', reify=True)
    elif request.cookies and 'lang' in request.cookies and\
            request.cookies['lang'] in available_languages:
        locale = request.cookies['lang']
    elif request.accept_language:
        locale = request.accept_language.best_match(available_languages)

    return locale


def destination_path(request):
    '''
        Returns absolute path of the translation destination

        :param pyramid.request.Request request: a request object

        :returns: A combined translation destination path
        :rtype: str
    '''
    package_name, filename = resolve_asset_spec(request.config.localize.translation.destination)

    if package_name is None:  # absolute filename
        directory = filename
    else:
        __import__(package_name)
        package = sys.modules[package_name]
        directory = os.path.join(package_path(package), filename)
    return directory


def dummy_autotranslate(msgid, domain=None, default=None, mapping=None):
    '''
        Method that simulate autotranslate

        :param str msgid: Message or message id
        :param str domain: Translation domain
        :param str default: Default message
        :param dict mapping: Mapping dictionary for message variables

        :returns: *translated* string
        :rtype: str

    '''
    # Try to return defaults first:
    tstr = None
    if default:
        tstr = default
    else:
        tstr = msgid

    if mapping and tstr:
        def replace(match):
            whole, param1, param2 = match.groups()
            return text_type(mapping.get(param1 or param2, whole))
        tstr = _interp_regex.sub(replace, tstr)

    return tstr



# -*- coding: utf-8 -*-


def language(field):
    def predicate(info, request):
        '''
            Checks whether language is one of the defaults
        '''
        if field in info['match'] and info['match'][field] in request.config.localize.locales.available:
            return True
        return False
    return predicate

language.__text__ = 'language predicate, to determine allowed languages in route'



# -*- coding: utf-8 -*-

from sqlalchemy import Column
from sqlalchemy import Sequence
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import Unicode
from sqlalchemy import DateTime
from sqlalchemy import func

from pyramid_basemodel import Base


class Language(Base):

    '''
        languages table model definition
    '''

    __tablename__ = 'languages'

    id = Column(Integer, Sequence(__tablename__ + '_sq'), primary_key=True)
    updated_at = Column(DateTime, default=func.now(), onupdate=func.now(), nullable=False)
    name = Column(Unicode(45), nullable=False)
    native_name = Column(Unicode(45), nullable=False)
    language_code = Column(String(2), unique=True, nullable=False)  # ISO 639-1 (Alpha2)

    def __unicode__(self):  # pragma: no cover
        '''
            Language to unicode conversion
        '''
        return self.name

    def __str__(self):  # pragma: no cover
        '''
            Language to string conversion
        '''
        return self.name.encode('utf8')



# -*- coding: utf-8 -*-
import sys
import os
import time
import logging
import subprocess


from pyramid.view import view_config
from pyramid.view import view_defaults
from pyramid.asset import resolve_asset_spec
from pyramid.path import package_path
from pyramid.httpexceptions import HTTPFound

from pyramid_localize.tools import destination_path
from pyramid_localize.tools import set_localizer

logger = logging.getLogger(__name__)


@view_defaults(permission='manage_translations')
class CatalogView(object):

    '''
            View class for catalog manipulation actions
    '''

    def __init__(request):
        '''Assign request'''
        self.request = request

    def _translation_file(self, language, domain, extension='po'):
        '''
            Creates a translation file path

            :param str language: two-letetr language code
            :param str domain: translation domain name
            :param str extension: translation file extension (po/mo)
        '''

        translation_destination = destination_path(self.request)
        return os.path.join(translation_destination, language, 'LC_MESSAGES', domain + '.' + extension)

    def _translation_template_path(self, spec):
        '''
            calculates path to translation template file

            :param str spec: either full path, or package related path
        '''
        # resolving possible asset spec to path (pyramid way):
        package_name, filename = resolve_asset_spec(spec)
        if package_name is None:  # absolute filename
            return filename
        else:
            __import__(package_name)
            package = sys.modules[package_name]
            return os.path.join(package_path(package), filename)

    @view_config(route_name='localize:index', renderer='pyramid_localize:resources/templates/index.mako')
    def index(self):
        '''
            Simple action listing domains, and its files of files with additional data, like compilation date, .po update date.

            :returns:

                .. code-block:: python

                    {
                        'language': {
                            'domain1': {
                                'po': 'modification time',
                                'pot': 'modification time',
                                'mo': 'modification time',
                                },
                            # more domains
                        },
                        # more languages
                    }
        '''
        translations = {}
        translation_sources = self.request.config.localize.translation.sources

        for language in self.request.config.localize.locales.available:
            translations[language] = {}
            for domain in translation_sources:
                translations[language][domain] = {
                    'po': None,
                    'pot': None,
                    'mo': None
                }
                po_file = self._translation_file(language, domain)
                if os.path.isfile(po_file):
                    translations[language][domain]['po'] = time.ctime(os.path.getmtime(po_file))

                pot_file = self._translation_template_path(translation_sources[domain])
                if os.path.isfile(pot_file):
                    translations[language][domain]['pot'] = time.ctime(os.path.getmtime(pot_file))

                mo_file = self._translation_file(language, domain, 'mo')
                if os.path.isfile(mo_file):
                    translations[language][domain]['mo'] = time.ctime(os.path.getmtime(mo_file))

        return {'translations': translations}

    @view_config(route_name='localize:update', renderer='pyramid_localize:resources/templates/index.mako')
    def update_catalog(self):
        '''
            This action updates or initializes translation catalogs (.po files) from their respective transaltion templates (.pot).
            This action is performed for every language defined within `localize.locales.available` config key.

            Redirects itself to **localize:index**.
        '''
        data = self.index()
        translation_sources = self.request.config.localize.translation.sources

        for domain in translation_sources:

            pot_file = self._translation_template_path(translation_sources[domain])

            if not os.path.isfile(pot_file):
                logger.critical('pot file for {domain} does not exists!'.format(domain=domain))

            for language in self.request.config.localize.locales.available:
                po_file = self._translation_file(language, domain)
                if os.path.isfile(po_file):
                    logger.debug('po file for {domain}, {language} exists, proceeding with update'.format(
                        domain=domain, language=language))
                    if subprocess.call([self.request.config.localize.pybabel,
                                        'update',
                                        '-l', language,
                                        '-i', pot_file,
                                        '-o', po_file,
                                        '--previous']):
                        logger.error('Error while trying to update {po} file!'.format(po=po_file))
                else:
                    logger.debug('po file for {domain}, {language} does not exists, proceeding with initialize'.format(
                        domain=domain, language=language))
                    if subprocess.call([self.request.config.localize.pybabel,
                                        'init',
                                        '-l', language,
                                        '-i', pot_file,
                                        '-o', po_file]):
                        logger.error('Error while trying to update {po} file!'.format(po=po_file))

        return HTTPFound(location=self.request.route_url('localize:index'))

    @view_config(route_name='localize:compile', renderer='pyramid_localize:resources/templates/index.mako')
    def compile_catalog(self):
        '''
            This action compiles all translation files (.po) for every language defined into .mo file that's used by gettext.

            redirects to **localize:index**.
        '''
        data = self.index()
        translation_sources = self.request.config.localize.translation.sources

        for domain in translation_sources:

            for language in self.request.config.localize.locales.available:
                po_file = self._translation_file(language, domain)
                mo_file = self._translation_file(language, domain, 'mo')
                if os.path.isfile(po_file):
                    logger.debug('po file for {domain}, {language} exists, proceeding with compilation'.format(
                        domain=domain, language=language))
                    if subprocess.call([self.request.config.localize.pybabel,
                                        'compile',
                                        '-l', language,
                                        '-i', po_file,
                                        '-o', mo_file,
                                        '--use-fuzzy']):
                        logger.error('Error while trying to compile {po} file!'.format(po=po_file))

        return HTTPFound(location=self.request.route_url('localize:index'))

    @view_config(route_name='localize:reload', xhr='True', renderer='json')
    @view_config(route_name='localize:reload', renderer='pyramid_localize:resources/templates/index.mako')
    def reload_catalog(self):
        '''
            Reloads transation caalog for application it's run in.

            .. note::
                To see how is this happening, you might want to see :func:`~pyramid_localize.tools.set_localizer`

            :returns:

                Only for xhr requests:

                .. code-block:: python

                    {
                        'status': True,
                        'msg': 'Localizators has been reloaded' # translated
                    }

                non xhr requests: Redirects to **localize:index**.


        '''

        set_localizer(self.request, True)

        if self.request.is_xhr:
            return {'status': True, 'msg': self.request._('Localizators has been reloaded')}

        return HTTPFound(location=self.request.route_url('localize:index'))



# -*- coding: utf-8 -*-

from pyramid.events import subscriber
from pyramid.events import BeforeRender
from pyramid.events import NewRequest

from pyramid_localize.tools import dummy_autotranslate


@subscriber(BeforeRender)
def global_renderer(event):
    '''
        Subscriber, which extends variables available for renderer with translator method, and localizer object
    '''
    request = event['request']
    try:
        event['_'] = request._
    except AttributeError:
        event['_'] = dummy_autotranslate


@subscriber(NewRequest)
def add_localizer(event):
    '''
        We add localzer for each new request (we use tools.set_localizer method, as to not repeat yourself)
    '''

    event.request._ = dummy_autotranslate



# -*- coding: utf-8 -*-

try:  # pragma: no cover
    import babel
except ImportError:  # pragma: no cover
    babel = False

from tzf.pyramid_yml import config_defaults

from pyramid_localize import tools
from pyramid_localize.request import locale
from pyramid_localize.request import database_locales
from pyramid_localize.request import locales
from pyramid_localize.request import locale_id

__version__ = '0.0.2'


def includeme(configurator):
    '''
        i18n includeme action
    '''

    # TODO: Find a better way to run other stuff than translation methods
    configuration = configurator.registry['config'].get('localize')
    # let's check if we have any configuration, or not
    if babel:
        configurator.scan('pyramid_localize.subscribers.i18n')
        if configuration:
            # once user allowed for localization, lets set up default values!
            config_defaults(configurator, 'pyramid_localize:config')

            configurator.set_locale_negotiator(tools.locale_negotiator)
            translation_dirs = configuration.translation.dirs
            # if it's not a list, lets make it a list. This is to allow creating both single, and list-like config entry
            if not isinstance(translation_dirs, list):
                translation_dirs = [translation_dirs]
            configurator.add_translation_dirs(*translation_dirs)
            # let scan all subscribers
            configurator.scan('pyramid_localize.views')

            configurator.add_route(name='localize:index', pattern='catalog')
            configurator.add_route(name='localize:update', pattern='catalog/update')
            configurator.add_route(name='localize:compile', pattern='catalog/compile')
            configurator.add_route(name='localize:reload', pattern='catalog/reload')

            # getting requests methods
            configurator.add_request_method(locale, name='locale', reify=True)
            configurator.add_request_method(database_locales, name='_database_locales', reify=True)
            configurator.add_request_method(locales, name='locales')
            configurator.add_request_method(locale_id, name='locale_id', reify=True)
    else:
        # including fake subscribers
        configurator.scan('pyramid_localize.subscribers.fake')



# -*- coding: utf-8 -*-

import os
import re
from setuptools import setup, find_packages

here = os.path.dirname(__file__)
with open(os.path.join(here, 'pyramid_localize', '__init__.py')) as v_file:
    package_version = re.compile(r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)


def read(fname):
    return open(os.path.join(here, fname)).read()

requirements = [
    'pyramid_basemodel',
    'tzf.pyramid_yml >=0.2',
    'pyramid'
]

test_requires = [
    'WebTest',
    'nose',
    'coverage',
    'mock'
]

extras_require = {
    'docs': ['sphinx', 'sphinx_bootstrap_theme'],
    'tests': test_requires,
    'babel': ['Babel >= 0.9']
}

setup(
    name='pyramid_localize',
    version=package_version,
    description='Package to provide translation methods for pyramid, and means to reload translations without stopping the application',
    long_description=(
        read('README.rst')
        + '\n\n' +
        read('CHANGES.rst')
    ),
    keywords='python template',
    author='Grzegorz Sliwinski',
    author_email='username: fizyk, domain: fizyk.net.pl',
    url='https://github.com/fizyk/pyramid_routing',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=find_packages(),
    install_requires=requirements,
    tests_require=test_requires,
    test_suite='tests',
    include_package_data=True,
    zip_safe=False,
    message_extractors={'pyramid_localize': [
                          ('**.py', 'python', None),
                          ('resources/templates/**.mako', 'mako', None),
                          ('resources/static/**', 'ignore', None)]},
    extras_require=extras_require,
)



# -*- coding: utf-8 -*-

import unittest
from mock import Mock

from pyramid.request import Request

from pyramid_localize.routing.predicates import language


class PredicatesTests(unittest.TestCase):

    def setUp(self):

        self.predicate = language('_LOCALE_')

    def _makeRequest(self, environ=None):
        if environ is None:
            environ = {}
        request = Request(environ)
        request.config = Mock()
        mock_configuration = {
            'localize.locales.available': ['en', 'pl', 'de', 'cz']}
        request.config.configure_mock(**mock_configuration)
        return request

    def test_positive(self):
        '''predicate:positive'''
        mock_info = {'match': {'_LOCALE_': 'en'}}

        request = self._makeRequest()
        self.assertTrue(self.predicate(mock_info, request))

    def test_negative(self):
        '''predicate:negative'''
        mock_info = {'match': {'_LOCALE_': 'fr'}}

        request = self._makeRequest()
        self.assertFalse(self.predicate(mock_info, request))

    def test_no_key(self):
        '''predicate:begative no key'''
        mock_info = {'match': {}}

        request = self._makeRequest()
        self.assertFalse(self.predicate(mock_info, request))



# -*- coding: utf-8 -*-

import unittest
from mock import Mock


from pyramid import testing
from pyramid.request import Request
from pyramid.events import BeforeRender
from pyramid.events import NewRequest
from pyramid.i18n import Localizer

# for this tests, these will be imported internally by pyramid's config
# from pyramid_localize.subscribers.i18n import global_renderer
# from pyramid_localize.subscribers.i18n import add_localizer
# from pyramid_localize.subscribers.fake import global_renderer
# from pyramid_localize.subscribers.fake import add_localizer


class SubscribersTests(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def _makeRequest(self, environ=None):
        if environ is None:
            environ = {}
        request = Request(environ)
        request.registry = self.config.registry

        return request

    def _makeBeforeRender(self, system, val=None):
        from pyramid.events import BeforeRender
        return BeforeRender(system, val)

    def test_i18n_new_request(self):
        '''i18n:Test new request'''
        self.config.scan('pyramid_localize.subscribers.i18n')
        request = self._makeRequest()
        request.registry.notify(NewRequest(request))
        self.assertTrue(isinstance(request.localizer, Localizer))
        self.assertTrue(hasattr(request, '_'))

    def test_i18n_before_render(self):
        '''i18n:Test before render'''
        self.config.scan('pyramid_localize.subscribers.i18n')
        request = self._makeRequest()
        before_render_event = self._makeBeforeRender({'request': request}, {})
        request.registry.notify(before_render_event)
        self.assertTrue('localizer' in before_render_event)
        self.assertTrue('_' in before_render_event)

    def test_i18n_before_render_and_request(self):
        '''i18n:Test before render with new request'''
        self.config.scan('pyramid_localize.subscribers.i18n')
        request = self._makeRequest()
        request.registry.notify(NewRequest(request))
        before_render_event = self._makeBeforeRender({'request': request}, {})
        request.registry.notify(before_render_event)
        self.assertTrue('localizer' in before_render_event)
        self.assertTrue('_' in before_render_event)

    def test_fake_new_request(self):
        '''fakei18n:Test new request'''
        self.config.scan('pyramid_localize.subscribers.fake')
        request = self._makeRequest()
        request.registry.notify(NewRequest(request))
        self.assertTrue(hasattr(request, '_'))

    def test_fake_before_render(self):
        '''fakei18n:Test before render'''
        self.config.scan('pyramid_localize.subscribers.fake')
        request = self._makeRequest()
        request.registry.notify(NewRequest(request))
        before_render_event = self._makeBeforeRender({'request': request}, {})
        request.registry.notify(before_render_event)
        self.assertTrue('_' in before_render_event)

    def test_fake_before_render_norequest(self):
        '''fakei18n:Test before render'''
        self.config.scan('pyramid_localize.subscribers.fake')
        request = self._makeRequest()
        before_render_event = self._makeBeforeRender({'request': request}, {})
        request.registry.notify(before_render_event)
        self.assertTrue('_' in before_render_event)
